package com.hiramine.myexoplayerwithsurfaceview;

import androidx.appcompat.app.AppCompatActivity;

import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.MotionEvent;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.SeekBar;
import android.widget.TextView;

import com.google.android.exoplayer2.Format;
import com.google.android.exoplayer2.MediaItem;
import com.google.android.exoplayer2.Player;
import com.google.android.exoplayer2.SimpleExoPlayer;

import java.util.Locale;

public class MainActivity extends AppCompatActivity
{
	private final String m_strUri = "https://commondatastorage.googleapis.com/gtv-videos-bucket/sample/BigBuckBunny.mp4";
	// 再生動画として、commondatastorage.googleapis.com の gtv-videos-bucket の sample に置いてある「Big Buck Bunny」動画ファイルを使用。
	// Big Buck Bunny
	// 　Copyright (C) 2008 Blender Foundation | peach.blender.org Some Rights Reserved.
	// 　Creative Commons Attribution 3.0 license. https://peach.blender.org/

	private SurfaceView     m_surfaceviewVideo;
	private SimpleExoPlayer m_simpleexoplayer;
	private boolean         m_bIsPlayingWhenActivityIsPaused;
	private TextView        m_textviewPosition;
	private TextView        m_textviewDuration;
	private SeekBar         m_seekbar;
	private Handler         m_handlerUpdateProgress = new Handler( Looper.getMainLooper() );    // メインスレッド（＝UIスレッド）で実行させるHandler。

	private Runnable m_runnableUpdateProgress = new Runnable()
	{
		@Override
		public void run()
		{
			updateProgress();

			m_handlerUpdateProgress.postDelayed( this, 100 );            // 100ミリ秒（0.1秒）後に再度実行する
		}
	};

	private final Player.Listener m_playerlistener = new Player.Listener()
	{
		@Override
		public void onIsPlayingChanged( boolean isPlaying )
		{
			if( isPlaying )
			{
				// 動画を再生するビューのサイズの調整
				adjustVideoViewSize();

				// 動画長さの設定
				// 動画の長さは、exoplayer.getDuration()によって、単位がミリ秒で得られる。
				long lDurationMilliSec = m_simpleexoplayer.getDuration();
				// 動画の長さのテキストは、動画の長さを「MM:SS」形式で設定する。
				long lDurationSec = (int)Math.ceil( lDurationMilliSec / 1000 );
				long lMM          = lDurationSec / 60;
				long lSS          = lDurationSec % 60;
				m_textviewDuration.setText( String.format( Locale.US, "%02d:%02d", lMM, lSS ) );
				// シークバーの最大値は、動画の長さを単位を0.1秒で設定する（100で割って、0.1秒単位に変換）。
				m_seekbar.setMax( (int)( lDurationMilliSec / 100 ) );
			}
		}
	};

	@Override
	protected void onCreate( Bundle savedInstanceState )
	{
		super.onCreate( savedInstanceState );
		setContentView( R.layout.activity_main );

		// Viewの取得
		m_textviewPosition = findViewById( R.id.textview_position );
		m_textviewDuration = findViewById( R.id.textview_duration );
		m_seekbar = findViewById( R.id.seekbar );

		// 動画を再生するビューの取得
		m_surfaceviewVideo = findViewById( R.id.surfaceview );

		// SimpleExoPlayerの作成と設定
		SimpleExoPlayer.Builder playerbuilder = new SimpleExoPlayer.Builder( this );
		m_simpleexoplayer = playerbuilder.build();
		m_simpleexoplayer.addListener( m_playerlistener );

		// 動画を再生するビューとSimpleExoPlayerの紐づけ
		m_simpleexoplayer.setVideoSurfaceView( m_surfaceviewVideo );

		// MediaItemの作成とSimpleExoPlayerへのセット
		Uri uriVideo = Uri.parse( m_strUri );
		m_simpleexoplayer.setMediaItem( MediaItem.fromUri( uriVideo ) );

		// 再生の準備
		m_simpleexoplayer.prepare();
		// 再生の開始
		m_simpleexoplayer.play();
	}

	// 初回表示時、および、ポーズからの復帰時
	@Override
	protected void onResume()
	{
		super.onResume();

		// 再生位置表示の定期更新の開始
		m_handlerUpdateProgress.postDelayed( m_runnableUpdateProgress, 0 );

		if( m_bIsPlayingWhenActivityIsPaused )
		{    // Activity pause時に動画再生中なら、動画再生再開
			m_simpleexoplayer.play();
		}
	}

	// アクティビティがフォアグラウンドからバックグラウンドに追いやられた時
	@Override
	protected void onPause()
	{
		// 動画再生中かどうかの取得と保持
		m_bIsPlayingWhenActivityIsPaused = m_simpleexoplayer.isPlaying();
		// 動画再生の一時停止
		m_simpleexoplayer.pause();

		// 再生位置表示の定期更新の停止
		m_handlerUpdateProgress.removeCallbacks( m_runnableUpdateProgress );

		super.onPause();
	}

	// 動画を再生するビューのサイズの調整
	private void adjustVideoViewSize()
	{
		// 動画があるかの確認
		Format videoformat = m_simpleexoplayer.getVideoFormat();
		if( null == videoformat )
		{
			return;
		}

		// 動画のサイズ
		int iVideoWidth  = videoformat.width;
		int iVideoHeight = videoformat.height;

		// 動画を再生するビューの親のサイズ
		ViewGroup viewgroupVideo = (ViewGroup)m_surfaceviewVideo.getParent();
		int iViewgroupWidth  = viewgroupVideo.getWidth();
		int iViewgroupHeight = viewgroupVideo.getHeight();

		// 動画の縦横比を保持しつつ、動画を再生するビューの親の中で最大限引きのばした、幅と高さを求める
		ViewGroup.LayoutParams layoutparams = m_surfaceviewVideo.getLayoutParams();
		double dVideoRatio     = (float)iVideoWidth / iVideoHeight;
		double dViewgropuRatio = (float)iViewgroupWidth / iViewgroupHeight;
		if( dVideoRatio > dViewgropuRatio )
		{
			layoutparams.width = iViewgroupWidth;
			layoutparams.height = (int)( (double)layoutparams.width / dVideoRatio );
		}
		else
		{
			layoutparams.height = iViewgroupHeight;
			layoutparams.width = (int)( (double)layoutparams.height * dVideoRatio );
		}

		// 求めた幅と高さを、動画を再生するビューに設定
		m_surfaceviewVideo.setLayoutParams( layoutparams );
	}

	// 再生位置表示の更新
	private void updateProgress()
	{
		// 動画の再生位置は、exoplayer.getCurrentPosition()によって、単位がミリ秒で得られる。
		long lPositionMilliSec = m_simpleexoplayer.getCurrentPosition();

		// 動画の再生位置のテキストは、動画の再生位置を「MM:SS」形式で設定する。
		long lPositionSec = lPositionMilliSec / 1000;
		long lMM          = lPositionSec / 60;
		long lSS          = lPositionSec % 60;
		m_textviewPosition.setText( String.format( Locale.US, "%02d:%02d", lMM, lSS ) );

		// プログレス位置は、動画の再生位置を単位を0.1秒で設定する（100で割って、0.1秒単位に変換）。
		m_seekbar.setProgress( (int)( lPositionMilliSec / 100 ) );
	}
}